import 'core-js/modules/es.object.assign.js';
import { jsxs, jsx } from 'react/jsx-runtime';
import { Component } from 'react';
import { c as common, a as charts, P as PropTypes } from './index.esm2.js';
import 'core-js/modules/es.array.reduce.js';
import * as d3$1 from 'd3';
import d3_tip from 'd3-tip';
import 'core-js/modules/es.array.sort.js';
import '@qubejs/core';
import 'core-js/modules/es.promise.js';
import 'core-js/modules/es.regexp.to-string.js';
import 'core-js/modules/es.regexp.exec.js';
import 'core-js/modules/es.string.search.js';
import 'core-js/modules/es.regexp.constructor.js';
import 'core-js/modules/es.string.match.js';
import 'core-js/modules/es.string.replace.js';
import 'tslib';
import 'core-js/modules/es.string.trim.js';
import 'lodash/filter';
import 'moment-timezone';
import 'core-js/modules/es.array.iterator.js';
import 'core-js/modules/es.promise.finally.js';
import 'core-js/modules/es.string.starts-with.js';
import 'core-js/modules/web.dom-collections.iterator.js';
import '@reduxjs/toolkit';
import 'core-js/modules/es.array.includes.js';
import 'core-js/modules/es.string.includes.js';
import 'lodash/uniq';
import 'lodash/groupBy';
import '@emotion/react';
import '@mui/material/Button';
import '@mui/material/IconButton';
import '@mui/material/Typography';
import '@mui/material/Dialog';
import '@mui/material/DialogTitle';
import '@mui/material/DialogActions';
import '@mui/material/DialogContent';
import '@mui/material/Slide';
import '@mui/material/LinearProgress';
import '@mui/material/CircularProgress';
import '@mui/icons-material/SelectAll';
import '@mui/lab/Alert';
import '@mui/lab/AlertTitle';
import '@mui/material/Autocomplete';
import '@mui/material/Chip';
import '@mui/material/Box';
import '@mui/material';
import '@mui/material/TextField';
import '@mui/material/BottomNavigation';
import '@mui/material/BottomNavigationAction';
import '@mui/material/Breadcrumbs';
import '@mui/icons-material/NavigateNext';
import '@mui/material/ButtonGroup';
import '@mui/material/Tooltip';
import 'core-js/modules/es.symbol.description.js';
import '@mui/material/FormLabel';
import '@mui/material/FormControl';
import '@mui/material/FormControlLabel';
import '@mui/material/Checkbox';
import '@mui/material/FormGroup';
import '@mui/material/FormHelperText';
import 'react-html-parser';
import '@mui/material/Menu';
import '@mui/material/MenuItem';
import '@mui/material/ListItemIcon';
import '@mui/material/Divider';
import '@mui/material/List';
import '@mui/material/ListItem';
import '@mui/material/ListItemText';
import '@mui/x-date-pickers/DatePicker';
import '@mui/x-date-pickers/AdapterDateFns';
import '@mui/x-date-pickers/LocalizationProvider';
import '@mui/x-date-pickers/TimePicker';
import '@mui/x-date-pickers/DateTimePicker';
import '@mui/x-date-pickers/DesktopDatePicker';
import '@mui/x-date-pickers/MobileDatePicker';
import '@mui/material/Snackbar';
import '@mui/styles/withStyles';
import '@mui/icons-material/AttachFile';
import '@mui/icons-material/CloudUpload';
import '@mui/material/Fab';
import '@mui/material/Grid';
import '@mui/icons-material/Delete';
import '@mui/material/SnackbarContent';
import '@mui/styles';
import '@mui/icons-material/CheckCircle';
import '@mui/icons-material/Close';
import '@mui/icons-material/Error';
import '@mui/icons-material/Info';
import '@mui/icons-material/Warning';
import '@mui/material/Popover';
import 'material-ui-popup-state';
import 'immutability-helper';
import '@mui/material/ListItemButton';
import '@mui/material/Collapse';
import '@mui/material/Avatar';
import '@mui/material/Select';
import '@mui/material/OutlinedInput';
import '@mui/material/Input';
import '@mui/material/InputLabel';
import '@mui/material/Pagination';
import '@mui/material/AppBar';
import '@mui/material/Toolbar';
import '@mui/material/Badge';
import '@mui/material/Drawer';
import '@mui/icons-material/ExpandLess';
import '@mui/icons-material/ExpandMore';
import '@mui/material/Tabs';
import '@mui/material/Tab';
import '@mui/material/Radio';
import '@mui/material/RadioGroup';
import 'react-google-recaptcha';
import '@mui/material/ListSubheader';
import '@mui/material/Skeleton';
import '@mui/material/Slider';
import '@mui/lab/SpeedDial';
import '@mui/lab/SpeedDialIcon';
import '@mui/lab/SpeedDialAction';
import '@mui/material/Switch';
import 'react-redux';
import 'react-router-dom';
import 'lodash/sortBy';
import '@mui/lab/TreeView';
import '@mui/icons-material/ChevronRight';
import '@mui/lab/TreeItem';

const colorsMap = {
  default: d3$1.schemeBlues.slice().pop().slice(1),
  expense: d3$1.schemeRdPu.slice().pop().slice(1),
  income: d3$1.schemeGreens.slice().pop().slice(1)
};
class Base {
  constructor(element, {
    d3 = d3$1,
    chart = {},
    chartType
  } = {}, defaults = {}) {
    this.chartType = chartType;
    this.config = Object.assign({
      minWidth: 320,
      minHeight: 200,
      width: 'auto',
      height: 300,
      xValue: 'x',
      yValue: 'y',
      margin: {
        top: 10,
        left: 10,
        right: 10,
        bottom: 10
      }
    }, defaults, chart);
    this.d3 = d3;
    this.d3.tip = d3_tip;
    this.element = element;
  }
  getColorSet(setName) {
    return colorsMap[setName] || colorsMap.default;
  }
  setConfig(config) {
    this.config = Object.assign(Object.assign({}, this.config), config);
  }
  init() {}
  update() {}
  draw() {}
  getValuesFromSeries(data) {
    const {
      series
    } = this.config;
    const allData = data.map(item => {
      return series.map(ser => {
        return item[ser.yValue] || 0;
      });
    });
    let allValues = [];
    if (allData.length > 0) {
      allValues = allData.reduce((a, b) => a.concat(b));
    }
    return allValues;
  }
  getWidth() {
    const element = this.element;
    let {
      width
    } = this.config;
    const {
      height,
      minWidth,
      maxWidth,
      margin
    } = this.config;
    if (width === 'auto') {
      width = element.offsetWidth >= minWidth ? element.offsetWidth : minWidth;
    } else if (element.offsetWidth < width) {
      width = element.offsetWidth;
    }
    const innerWidth = width - margin.left - margin.right;
    const innerHeight = height - margin.top - margin.bottom;
    return {
      width,
      height,
      minWidth,
      maxWidth,
      innerWidth,
      innerHeight,
      margin
    };
  }
  dispose() {
    this.element.innerHTML = '';
  }
}

class ColumnChart extends Base {
  constructor(el, options = {}) {
    super(el, {
      chart: options,
      chartType: 'ColumnChart'
    }, {
      margin: {
        top: 20,
        bottom: 90,
        left: 60,
        right: 20
      }
    });
  }
  getConfig() {}
  init() {}
  resetWidth() {
    const d3 = this.d3;
    d3.select(this.element).select('svg').attr('width', 'auto');
  }
  refresh() {
    this.update();
  }
  update() {
    const d3 = this.d3;
    const element = this.element;
    const {
      colorSet,
      xAxis = {},
      yAxis,
      tooltip = {},
      legend = {}
    } = this.config;
    const {
      labelWidth: legendLabelWidth = 80,
      enable: legendEnabled = true
    } = legend;
    const {
      format,
      ticks = 5
    } = yAxis || {};
    const {
      labelWidth = 44
    } = xAxis;
    const {
      formatter: formatTooltip = d => d[xValue]
    } = tooltip;
    const colorSetDefault = this.getColorSet(colorSet);
    const {
      xValue,
      series = []
    } = this.config;
    const columnSeries = series.filter(t => !t.type || t.type === 'Column');
    const lineSeries = series.filter(t => t.type === 'Line');
    d3.select(element).select('svg').attr('width', 'auto');
    const {
      width,
      height,
      innerWidth,
      innerHeight
    } = this.getWidth();
    d3.select(element).select('svg').attr('width', width).attr('height', height);
    if (!this.data) {
      return;
    }
    const data = this.data;
    const allValues = this.getValuesFromSeries(this.data);
    const x = d3.scaleBand().range([0, innerWidth]).domain(data.map(d => d[xValue])).paddingInner(0.2).paddingOuter(0.1);
    this.x = x;
    let minAll = d3.min(allValues, d => d);
    if (minAll > 0) {
      minAll = 0;
    }
    const y = d3.scaleLinear().domain([minAll * 1.005, (d3.max(allValues, d => d) || 100) * 1.005]).range([innerHeight, 0]);
    this.y = y;
    const g = this.gContainer;
    this.tooltip.html(formatTooltip);
    const xAxisCall = d3.axisBottom(x);
    this.xAxisGroup.call(xAxisCall).selectAll('text').attr('y', '10').attr('x', '-5').attr('text-achor', 'end').attr('transform', 'rotate(0)').style('display', function (d, i) {
      const num = data.length / (innerWidth / (labelWidth + 20));
      const everyItem = Math.ceil(num);
      return i % everyItem ? 'none' : 'initial';
    });
    const yAxisCall = d3.axisLeft(y).ticks(ticks).tickFormat(d => format ? format(d) : d);
    this.yAxisGroup.call(yAxisCall);
    let slice = this.columnArea.selectAll('.group').data(data, d => d[xValue]);
    slice.exit().remove();
    slice = slice.enter().append('g').merge(slice).attr('class', 'group').on('mouseover', this.tooltip.show).on('mouseout', this.tooltip.hide);
    const rects = slice.selectAll('rect').data(d => {
      return columnSeries.map((ser, idx) => {
        return {
          orig: ser,
          x: d[ser.xValue],
          y: d[ser.yValue],
          color: ser.color || colorSetDefault[idx]
        };
      });
    });
    function getX(d, idx) {
      return x(d.x) + x.bandwidth(d.x) / columnSeries.length * idx;
    }
    function getWidth(d) {
      return x.bandwidth(d.x) / columnSeries.length;
    }
    lineSeries.forEach(item => {
      const line = d3.line().x(d => {
        return x(d[item.xValue]) + x.bandwidth(d.x) / columnSeries.length;
      }).y(d => {
        return y(d[item.yValue]);
      });
      g.selectAll(`.${item.name}`).transition(d3.transition().duration(750)).attr('d', line(data));
    });
    rects.exit().remove();
    if (legendEnabled) {
      const legRects = this.legend.selectAll('rect').data(series);
      const totalSeries = series.length;
      legRects.enter().append('rect').merge(legRects).attr('x', function (d, i) {
        return innerWidth - (totalSeries - i) * legendLabelWidth;
      }).attr('y', function (d, i) {
        return height - 70;
      }).attr('width', 10).attr('height', 10).style('fill', function (d) {
        const color = d.color;
        return color;
      });
      const legTexts = this.legend.selectAll('text').data(series);
      legTexts.enter().append('text').merge(legTexts).attr('x', function (d, i) {
        return innerWidth - (totalSeries - i) * legendLabelWidth + 16;
      }).attr('y', function (d, i) {
        return height - 70;
      }).attr('alignment-baseline', 'hanging').text(function (d) {
        const text = d.label || d.name;
        return text;
      });
    }
    rects.enter().append('rect').attr('fill', d => d.color).attr('y', d => y(0)).attr('height', d => 0).attr('width', getWidth).merge(rects).attr('x', getX).transition(d3.transition().duration(750)).attr('width', getWidth).attr('y', d => y(d.y)).attr('height', d => y(0) - y(d.y));
  }
  mousemoveTooltip(e) {}
  draw() {
    const element = this.element;
    const d3 = this.d3;
    const {
      yValue,
      xValue,
      margin,
      yAxisLabel,
      xAxisLabel,
      series = []
    } = this.config;
    const lineSeries = series.filter(item => item.type === 'Line');
    const {
      width,
      height,
      innerWidth,
      innerHeight
    } = this.getWidth();
    const svg = d3.select(element).append('svg').attr('class', 'chart-svg').attr('width', width).attr('height', height);
    const g = svg.append('g').attr('transform', `translate(${margin.left}, ${margin.top})`);
    this.gContainer = g;
    this.xAxisGroup = g.append('g').attr('class', 'x axis').attr('transform', `translate(0, ${innerHeight})`);
    this.yAxisGroup = g.append('g').attr('class', 'y axis');
    this.columnArea = g.append('g').attr('class', 'column');
    this.lineArea = g.append('g').attr('class', 'line');
    lineSeries.forEach(item => {
      this.lineArea.append('path').attr('class', item.name).attr('fill', 'none').attr('stroke', item.color).attr('stroke-width', '3px');
    });
    this.legend = g.append('g').attr('class', 'sq-chart-legend');
    this.tooltip = d3.tip().attr('class', 'sq-chart-tooltip');
    g.call(this.tooltip);
    if (!this.data) {
      return;
    }
    this.update();
  }
  dispose() {
    this.tooltip && this.tooltip.destroy();
    this.element.innerHTML = '';
  }
}

window.d3 = d3$1;
// const d3 = window.d3;
class PieChart extends Base {
  constructor(el, options = {}) {
    super(el, {
      chart: options,
      chartType: 'PieChart'
    }, {});
  }
  getConfig() {}
  dispose() {
    this.element.innerHTML = '';
  }
  update() {
    if (!this.data) {
      return;
    }
    const data = this.data;
    var {
      xValue,
      yValue,
      colorSet,
      margin
    } = this.config;
    this.svg.attr('width', 'auto');
    const {
      width,
      height
    } = this.getWidth();
    this.svg.attr('width', width).attr('height', height);
    const pie = d3$1.pie().padAngle(0.005).sort(null).value(d => {
      return d[yValue];
    });
    const arcs = pie(data);
    this.totalSum = d3$1.sum(data, function (d) {
      return d[yValue];
    });
    const catKeys = data.map(d => d[xValue]);
    const color2 = d3$1.scaleOrdinal().domain(catKeys).range(this.getColorSet(colorSet));
    this.rootGroup.attr('transform', `translate(${margin.left}, ${margin.top})`).selectAll('path').data(arcs).join('path').attr('fill', d => color2(d.data[xValue])).on('mouseenter', this.onMouseEnter.bind(this)).on('mouseleave', this.onMouseLeave.bind(this))
    // .attr('d', this.arc)
    .transition().duration(500).attrTween('d', d => {
      var i = d3$1.interpolate(d.startAngle + 0.1, d.endAngle);
      return t => {
        d.endAngle = i(t);
        return this.arc(d);
      };
    });
  }
  draw() {
    const element = this.element;
    const d3 = this.d3;
    var {
      margin
    } = this.config;
    var {
      innerWidth,
      innerHeight
    } = this.getWidth();
    const radius = Math.min(innerWidth, innerHeight) / 2;
    this.arc = d3.arc().innerRadius(radius * 0.4).outerRadius(radius * 0.9);
    this.svg = d3.select(element).append('svg').attr('viewBox', [-innerWidth / 2, -innerHeight / 2, innerWidth, innerHeight]);
    this.textPie = this.svg.append('g').attr('class', 'percentage').attr('transform', `translate(${margin.left}, ${margin.top})`).append('text');
    this.textPie.attr('class', 'sq-chart__pie-text').attr('font-family', "'Work sans', sans-serif").attr('font-size', 24).attr('font-weight', 900).attr('text-anchor', 'middle').style('opacity', '0').text('0%');
    const g = this.svg.append('g');
    this.rootGroup = g;
    this.legendGroup = g.append('g').attr('class', 'sq-chart__legend').attr('transform', `translate(${innerWidth / 2 - 20}, ${50})`);
    this.tooltip = d3.create('div').attr('class', 'sq-chart-tooltip').html(`
    <div class="sq-chart-tooltip__data">
      
    </div>
  `);
    element.appendChild(this.tooltip.node());
    if (!this.data) {
      return;
    }
    this.update();
  }
  onMouseEnter(e, d) {
    const {
      tooltip: tooltipConfig = {},
      xValue,
      yValue,
      margin
    } = this.config;
    const {
      formatter
    } = tooltipConfig;
    var {
      innerWidth,
      innerHeight
    } = this.getWidth();
    const tooltip = this.tooltip;
    const x = this.arc.centroid(d)[0] + innerWidth / 2;
    const y = this.arc.centroid(d)[1] + innerHeight / 2;
    tooltip.style('opacity', 1);
    tooltip.style('transform', `translate(calc( -50% + ${x + margin.left}px), calc(-100% + ${y + margin.top}px))`);
    const data = formatter && formatter(e, d) || d.data[xValue];
    tooltip.select('.sq-chart-tooltip__data').html(data);
    this.textPie.text(d3$1.format('.0%')(d.data[yValue] / this.totalSum)).style('opacity', 1);
  }
  onMouseLeave() {
    const tooltip = this.tooltip;
    tooltip.style('opacity', 0);
    this.textPie.style('opacity', 0);
  }
}

class LineChart extends Base {
  constructor(el, options = {}) {
    super(el, {
      chart: options,
      chartType: 'LineChart'
    }, {
      margin: {
        top: 20,
        bottom: 90,
        left: 60,
        right: 20
      }
    });
  }
  getConfig() {}
  init() {}
  refresh() {
    this.update();
  }
  hideAllFocus() {
    this.config;
    this.focus.main.style('display', 'none');
    const tooltip = d3.select(this.element).select('.sq-chart-d-tooltip');
    tooltip.style('opacity', '0');
  }
  showAllFocus() {
    const {
      series
    } = this.config;
    this.focus.main.style('display', null);
    series.forEach(ser => {});
  }
  update() {
    const d3 = this.d3;
    const vis = this;
    this.element;
    var {
      xValue,
      margin,
      series = [],
      colorSet,
      yAxis = {},
      xAxis = {},
      tooltip = {},
      legendLabelWidth = 100
    } = this.config;
    const {
      type: xAxisDataType
    } = xAxis;
    const {
      format: yAxisFormatter
    } = yAxis;
    const {
      formatter: tooltipFormatter = v => v
    } = tooltip;
    var {
      width,
      height,
      innerWidth,
      innerHeight
    } = this.getWidth();
    vis.t = d3.transition().duration(1000);
    vis.x = d3.scaleUtc().range([0, innerWidth]);
    vis.y = d3.scaleLinear().range([innerHeight, 0]);
    vis.data = vis.data.map(item => {
      return Object.assign({}, item, {
        [xValue]: xAxisDataType === 'date' && !(item[xValue] instanceof Date) ? new Date(item[xValue]) : item[xValue]
      });
    });
    let allValues = this.getValuesFromSeries(vis.data);
    vis.x.domain(d3.extent(vis.data, d => d[xValue]));
    vis.y.domain([d3.min(allValues, d => d) / 1.005, d3.max(allValues, d => d) * 1.005]);
    vis.svg.attr('class', 'chart-svg').attr('width', width).attr('height', height);
    vis.g.attr('transform', `translate(${margin.left}, ${margin.top})`);
    vis.xAxis.attr('transform', `translate(0, ${innerHeight})`);
    // update axes
    vis.xAxisCall.scale(vis.x);
    vis.xAxis.transition(vis.t).call(vis.xAxisCall);
    vis.yAxisCall.scale(vis.y);
    vis.yAxis.transition(vis.t).call(vis.yAxisCall.tickFormat(yAxisFormatter));

    // clear old tooltips
    vis.g.select('.focus').remove();
    vis.g.select('.overlay').remove();

    /******************************** Tooltip Code ********************************/
    vis.focus = {};
    vis.focus.main = vis.g.append('g').attr('class', `focus`).style('display', 'none');
    vis.focus.main.append('line').attr('class', 'x-hover-line hover-line').attr('y1', 0).attr('y2', innerHeight);
    series.forEach(ser => {
      vis.focus[ser.name] = vis.focus.main.append('g').attr('class', `circle-data ${ser.name}`);
      vis.focus[ser.name].append('circle').attr('r', 7.5).attr('class', `sq-line-chart-circle`).style('stroke', ser.color);
      vis.focus[ser.name].append('text').attr('x', 15).attr('dy', '.31em').attr('class', `sq-line-chart-t-text`);
    });
    vis.g.append('rect').attr('class', 'sq-chart-overlay').attr('width', innerWidth).attr('height', innerHeight).on('mouseover', () => this.showAllFocus()).on('mouseout', () => this.hideAllFocus()).on('mousemove', mousemove);
    function mousemove(e) {
      const x0 = vis.x.invert(d3.pointer(e)[0]);
      const i = vis.bisectDate(vis.data, x0, 1);
      const d0 = vis.data[i > 0 ? i - 1 : 0];
      const d1 = vis.data[i >= vis.data.length ? vis.data.length - 1 : i];
      const d = x0 - d0[xValue] > d1[xValue] - x0 ? d1 : d0;
      vis.focus.main.attr('transform', `translate(${vis.x(d[xValue])},0 )`);
      vis.focus.main.select('.x-hover-line').attr('y2', innerHeight);
      const tooltip = d3.select(vis.element).select('.sq-chart-d-tooltip');
      tooltip.html(tooltipFormatter(e, d, series));
      tooltip.style('left', `${vis.x(d[xValue])}px`);
      tooltip.style('opacity', `1`);
      tooltip.style('top', `0px`);
      series.forEach(ser => {
        vis.focus[ser.name].select('circle').attr('cy', vis.y(d[ser.yValue]));
        vis.focus[ser.name].select('text').attr('y', vis.y(d[ser.yValue]));
        vis.focus[ser.name].select('text').html(tooltipFormatter(d[ser.yValue], d));
      });
    }

    /******************************** Legend Code ********************************/
    const legRects = this.legend.selectAll('rect').data(series);
    const totalSeries = series.length;
    legRects.enter().append('rect').merge(legRects).attr('x', function (d, i) {
      return innerWidth - (totalSeries - i) * legendLabelWidth;
    }).attr('y', function (d, i) {
      return height - 70;
    }).attr('width', 10).attr('height', 10).style('fill', function (d) {
      var color = d.color;
      return color;
    });
    const legTexts = this.legend.selectAll('text').data(series);
    legTexts.enter().append('text').merge(legTexts).attr('x', function (d, i) {
      return innerWidth - (totalSeries - i) * legendLabelWidth + 16;
    }).attr('y', function (d, i) {
      return height - 70;
    }).attr('alignment-baseline', 'hanging').text(function (d) {
      var text = d.label || d.name;
      return text;
    });
    /******************************** End Legend Code ********************************/

    // Path generator
    // Update our line path
    series.forEach(ser => {
      vis.line = d3.line()
      // .curve(d3.curveLinear)
      .x(d => {
        return !common.isNullOrUndefined(d[xValue]) ? vis.x(d[xValue]) : null;
      }).y(d => {
        return !common.isNullOrUndefined(d[ser.yValue]) ? vis.y(d[ser.yValue]) : null;
      });
      vis.g.select(`.line.${ser.name}`).attr('stroke', ser.color).attr('d', vis.line(vis.data));
    });
  }
  draw() {
    const vis = this;
    const element = this.element;
    const d3 = this.d3;
    var {
      xValue,
      margin,
      yAxisLabel,
      xAxisLabel,
      series = []
    } = this.config;
    this.getWidth();
    const svg = d3.select(element).append('svg');
    const g = svg.append('g');
    this.svg = svg;
    this.legend = g.append('g').attr('class', 'sq-chart-legend');
    vis.g = g;
    vis.parseTime = d3.timeParse('%d/%m/%Y');
    vis.formatTime = d3.timeFormat('%d/%m/%Y');
    // for tooltip
    vis.bisectDate = d3.bisector(d => d[xValue]).left;
    series.forEach(ser => {
      const {
        strokeWidth = '3px'
      } = ser;
      vis.g.append('path').attr('class', `line ${ser.name}`).attr('fill', 'none').attr('stroke', 'grey').attr('stroke-width', strokeWidth);
    });
    this.legend = g.append('g').attr('class', 'sq-chart-legend');
    this.tooltipEl = d3.select(element).append('div').attr('class', 'sq-chart-d-tooltip');
    // axis generators
    vis.xAxisCall = d3.axisBottom();
    vis.yAxisCall = d3.axisLeft().ticks(6);

    // axis groups
    vis.xAxis = vis.g.append('g').attr('class', 'x axis');
    vis.yAxis = vis.g.append('g').attr('class', 'y axis');
    if (!this.data) {
      return;
    }
    this.update();
  }
  dispose() {
    this.tooltip && this.tooltip.destroy();
    this.element.innerHTML = '';
  }
}

class AreaChart extends Base {
  constructor(el, options = {}) {
    super(el, {
      chart: options,
      chartType: 'AreaChart'
    }, {
      margin: {
        top: 20,
        bottom: 90,
        left: 60,
        right: 20
      }
    });
  }
  getConfig() {}
  init() {}
  refresh() {
    this.update();
  }
  hideAllFocus() {
    this.config;
    this.focus.main.style('display', 'none');
    const tooltip = d3$1.select(this.element).select('.sq-chart-d-tooltip');
    tooltip.style('opacity', '0');
  }
  showAllFocus() {
    const {
      series
    } = this.config;
    this.focus.main.style('display', null);
    series.forEach(ser => {});
  }
  resetWidth() {
    this.svg.attr('class', 'chart-svg').attr('width', 'auto');
  }
  update() {
    const d3 = this.d3;
    const vis = this;
    this.element;
    const {
      xValue,
      margin,
      series = [],
      colorSet,
      xAxis = {},
      yAxis = {},
      tooltip = {},
      legendLabelWidth = 100
    } = this.config;
    const {
      format: xAxisFormatter,
      type: xAxisDataType
    } = xAxis;
    const {
      labelWidth = 80,
      format: yAxisFormatter,
      minValue: yAxisMinValue
    } = yAxis;
    const {
      formatter: tooltipFormatter = v => v
    } = tooltip;
    vis.svg.attr('class', 'chart-svg').attr('width', 'auto');
    const {
      width,
      height,
      innerWidth,
      innerHeight
    } = this.getWidth();
    vis.t = d3.transition().duration(1000);
    vis.x = d3.scaleTime().range([0, innerWidth]);
    vis.y = d3.scaleLinear().range([innerHeight, 0]);
    vis.data = vis.data.map(item => {
      return Object.assign(Object.assign({}, item), {
        [xValue]: xAxisDataType === 'date' && !(item[xValue] instanceof Date) ? new Date(item[xValue]) : item[xValue]
      });
    });
    const allValues = this.getValuesFromSeries(vis.data);
    const scale = 1.005;
    let minValue = d3.min(allValues, d => d) * scale;
    const maxValue = d3.max(allValues, d => d) * scale;
    if (yAxisMinValue !== undefined && minValue > yAxisMinValue) {
      minValue = yAxisMinValue;
    }
    vis.x.domain(d3.extent(vis.data, d => d[xValue]));
    vis.y.domain([minValue, maxValue]);
    vis.svg.attr('class', 'chart-svg').attr('width', width).attr('height', height);
    vis.g.attr('transform', `translate(${margin.left}, ${margin.top})`);
    vis.xAxis.attr('transform', `translate(0, ${innerHeight})`);
    // update axes
    const xTicksCount = innerWidth / labelWidth;
    vis.xAxisCall.ticks(xTicksCount > vis.data.length ? vis.data.length + 1 : xTicksCount);
    vis.xAxisCall.scale(vis.x);
    vis.xAxis.transition(vis.t).call(vis.xAxisCall.tickFormat(xAxisFormatter));
    vis.yAxisCall.scale(vis.y);
    vis.yAxis.transition(vis.t).call(vis.yAxisCall.tickFormat(yAxisFormatter));
    // clear old tooltips
    vis.g.select('.focus').remove();
    vis.g.select('.overlay').remove();
    /******************************** Tooltip Code ********************************/
    vis.focus = {};
    vis.focus.main = vis.g.append('g').attr('class', `focus`).style('display', 'none');
    vis.focus.main.append('line').attr('class', 'x-hover-line hover-line').attr('y1', 0).attr('y2', innerHeight);
    // vis.focus.main.append('rect').attr('class', 'x-tooltip').attr('y1', 0).attr('y2', innerHeight)
    // vis.focus.main.select('rect').attr('fill', '#fff').append('text').texchart('Test');
    series.forEach(ser => {
      vis.focus[ser.name] = vis.focus.main.append('g').attr('class', `circle-data ${ser.name}`);
      vis.focus[ser.name].append('circle').attr('r', 5.5).attr('class', `sq-line-chart-circle`).style('stroke', ser.color);
      // vis.focus[ser.name].append('text').attr('x', 15).attr('dy', '.31em').attr('class', `sq-line-chart-t-text`);
    });
    vis.overlay.attr('width', innerWidth).attr('height', innerHeight).on('mouseover', () => this.showAllFocus()).on('mouseout', () => this.hideAllFocus()).on('mousemove', mousemove);
    function mousemove(e) {
      const x0 = vis.x.invert(d3.pointer(e)[0]);
      const bisectDate = d3.bisector(function (d, x) {
        return d[xValue] - x;
      }).left;
      const i = bisectDate(vis.data, x0);
      const d0 = vis.data[i > 0 ? i - 1 : 0];
      const d1 = vis.data[i >= vis.data.length ? vis.data.length - 1 : i];
      const d = x0 - d0[xValue] > d1[xValue] - x0 ? d1 : d0;
      vis.focus.main.attr('transform', `translate(${vis.x(d[xValue])},0 )`);
      vis.focus.main.select('.x-hover-line').attr('y2', innerHeight);
      const tooltip = d3.select(vis.element).select('.sq-chart-d-tooltip');
      tooltip.html(tooltipFormatter(e, d, series));
      tooltip.style('left', `${vis.x(d[xValue])}px`);
      tooltip.style('opacity', `1`);
      tooltip.style('top', `0px`);
      series.forEach(ser => {
        vis.focus[ser.name].select('circle').attr('cy', vis.y(d[ser.yValue] || 0)).style('stroke', d[ser.yValue] < 0 ? ser.negativeColor : ser.color);
        // vis.focus[ser.name].select('text').attr('y', vis.y(d[ser.yValue]));
        // vis.focus[ser.name].select('text').text(tooltipFormatter(d[ser.yValue]));
      });
    }
    /******************************** Legend Code ********************************/
    const legRects = this.legend.selectAll('rect').data(series);
    const totalSeries = series.length;
    legRects.enter().append('rect').merge(legRects).attr('x', function (d, i) {
      return innerWidth - (totalSeries - i) * legendLabelWidth;
    }).attr('y', function (d, i) {
      return height - 70;
    }).attr('width', 10).attr('height', 10).style('fill', function (d) {
      const color = d.color;
      return color;
    });
    const legTexts = this.legend.selectAll('text').data(series);
    legTexts.enter().append('text').merge(legTexts).attr('x', function (d, i) {
      return innerWidth - (totalSeries - i) * legendLabelWidth + 16;
    }).attr('y', function (d, i) {
      return height - 70;
    }).attr('alignment-baseline', 'hanging').text(function (d) {
      const text = d.label || d.name;
      return text;
    });
    /******************************** End Legend Code ********************************/
    const pos_height = vis.y(0);
    const neg_height = vis.y(minValue);
    vis.defs.select(`#clip_pos`).select('rect').attr('x', 0).attr('y', 0).attr('width', innerWidth).attr('height', pos_height > 0 ? pos_height : 0);
    vis.defs.select(`#clip_neg`).select('rect').attr('x', 0).attr('y', vis.y(maxValue > 0 ? 0 : maxValue)).attr('width', innerWidth).attr('height', neg_height > 0 ? neg_height : 0);
    // Path generator
    // Update our line path
    series.forEach(ser => {
      vis.line = d3.line()
      // .curve(d3.curveCardinal)
      .x(d => vis.x(d[xValue])).y(d => vis.y(d[ser.yValue] || 0));
      vis.area = d3.area()
      // .curve(d3.curveCardinal)
      .x(d => vis.x(d[xValue])).y0(d => vis.y(minValue > 0 ? minValue : 0)).y1(d => vis.y(d[ser.yValue] || 0));
      vis.g.select(`.area.p-${ser.name}`).attr('stroke', 'none').transition(vis.t).attr('d', vis.area(vis.data)).style('clip-path', 'url(#clip_pos)');
      vis.g.select(`.area.n-${ser.name}`).attr('stroke', 'none').transition(vis.t).attr('d', vis.area(vis.data)).style('clip-path', 'url(#clip_neg)');
      vis.g.select(`.line.${ser.name}`).attr('stroke', ser.color).transition(vis.t).attr('d', vis.line(vis.data)).style('clip-path', 'url(#clip_pos)');
      vis.g.select(`.line-n.${ser.name}`).attr('stroke', ser.negativeColor).transition(vis.t).attr('d', vis.line(vis.data)).style('clip-path', 'url(#clip_neg)');
    });
  }
  draw() {
    const vis = this;
    const element = this.element;
    const d3 = this.d3;
    const {
      xValue,
      margin,
      yAxisLabel,
      xAxisLabel,
      series = [],
      xAxis = {}
    } = this.config;
    const {
      width,
      height,
      innerWidth,
      innerHeight
    } = this.getWidth();
    const svg = d3.select(element).append('svg');
    const g = svg.append('g');
    this.svg = svg;
    this.tooltipEl = d3.select(element).append('div').attr('class', 'sq-chart-d-tooltip');
    this.legend = g.append('g').attr('class', 'sq-chart-legend');
    vis.g = g;
    vis.parseTime = d3.timeParse('%d/%m/%Y');
    vis.formatTime = d3.timeFormat('%d/%m/%Y');
    // for tooltip
    vis.bisectDate = d3.bisector(d => d[xValue]).left;
    vis.defs = svg.append('defs');
    series.forEach(ser => {
      const {
        strokeWidth = '3px'
      } = ser;
      vis.g.append('path').attr('class', `area n-${ser.name}`).attr('fill', `url(#gradN${ser.name})`).attr('stroke', 'grey').attr('stroke-width', strokeWidth);
      vis.g.append('path').attr('class', `area p-${ser.name}`).attr('fill', `url(#grad${ser.name})`).attr('stroke', 'grey').attr('stroke-width', strokeWidth);
      vis.g.append('path').attr('class', `line ${ser.name}`).attr('fill', 'none').attr('stroke', 'grey').attr('stroke-width', strokeWidth);
      vis.g.append('path').attr('class', `line-n ${ser.name}`).attr('fill', 'none').attr('stroke', 'grey').attr('stroke-width', strokeWidth);
      vis.lg = vis.defs.append('linearGradient').attr('id', `grad${ser.name}`).attr('x1', '0%').attr('x2', '0%').attr('y1', '0%').attr('y2', '100%');
      vis.lg.append('stop').attr('offset', '0%').style('stop-color', ser.color).style('stop-opacity', 0.4);
      vis.lg.append('stop').attr('offset', '100%').style('stop-color', 'white').style('stop-opacity', 0.4);
      vis.lgN = vis.defs.append('linearGradient').attr('id', `gradN${ser.name}`).attr('x1', '0%').attr('x2', '0%').attr('y1', '0%').attr('y2', '100%');
      vis.lgN.append('stop').attr('offset', '0%').style('stop-color', 'white').style('stop-opacity', 0.4);
      vis.lgN.append('stop').attr('offset', '100%').style('stop-color', ser.negativeColor).style('stop-opacity', 0.4);
    });
    vis.defs.append('clipPath').attr('id', `clip_pos`).append('rect').attr('x', 0).attr('y', 0).attr('width', innerWidth);
    vis.defs.append('clipPath').attr('id', `clip_neg`).append('rect').attr('x', 0).attr('y', 200).attr('width', innerWidth);
    this.legend = g.append('g').attr('class', 'sq-chart-legend');
    // axis generators
    vis.xAxisCall = d3.axisBottom();
    vis.yAxisCall = d3.axisLeft().ticks(6);
    vis.overlay = vis.g.append('rect').attr('class', 'sq-chart-overlay');
    // axis groups
    vis.xAxis = vis.g.append('g').attr('class', 'x axis');
    vis.yAxis = vis.g.append('g').attr('class', 'y axis');
    if (!this.data) {
      return;
    }
    this.update();
  }
  dispose() {
    this.tooltip && this.tooltip.destroy();
    this.element.innerHTML = '';
  }
}

const _chartMap = {
  d3: {
    Column: ColumnChart,
    Pie: PieChart,
    Line: LineChart,
    Area: AreaChart
  }
};
class ChartFactory {
  constructor({
    provider = 'd3'
  } = {}) {
    this.provider = provider;
  }
  get() {
    return _chartMap[this.provider];
  }
}

charts.set(new ChartFactory().get());
class CustomChart extends Component {
  constructor(props) {
    super(props);
    this.state = {};
    this.resizeChart = this.resizeChart.bind(this);
  }
  componentWillUnmount() {
    this.chart && this.chart.dispose();
    window.removeEventListener('resize', this.resizeChart);
  }
  componentDidMount() {
    this.renderChart();
    window.addEventListener('resize', this.resizeChart);
  }
  resizeChart() {
    var _a, _b;
    if (this.resizing) {
      return;
    }
    ((_a = this.chart) === null || _a === void 0 ? void 0 : _a.resetWidth) && ((_b = this.chart) === null || _b === void 0 ? void 0 : _b.resetWidth());
    this.resizing = window.setTimeout(() => {
      this.chart && this.chart.update();
      this.resizing = undefined;
    }, 500);
  }
  renderChart() {
    const {
      data,
      chartConfig = {},
      type,
      colorsSet = 'default'
    } = this.props;
    const ChartTypesMap = charts.get();
    const ChartTypeRender = ChartTypesMap[type] || ChartTypesMap.Pie;
    this.chart = new ChartTypeRender(this.chartContainer, Object.assign(Object.assign({}, chartConfig), {
      colorsSet
    }));
    this.chart.data = data;
    this.chart.draw();
  }
  componentDidUpdate(prevProps) {
    if (this.props.data !== prevProps.data || this.props.type !== prevProps.type) {
      if (!this.chart) {
        this.renderChart();
      } else {
        const {
          data,
          chartConfig = {},
          colorsSet = 'default'
        } = this.props;
        this.chart.setConfig(Object.assign(Object.assign({}, chartConfig), {
          colorsSet
        }));
        this.chart.data = data;
        this.chart.update();
      }
    }
  }
  render() {
    const {
      className = '',
      data = [],
      noDataMessage = 'No data found'
    } = this.props;
    return jsxs("div", {
      className: `sq-chart ${className}`,
      children: [data.length === 0 && jsx("div", {
        className: "sq-chart--no-data",
        children: noDataMessage
      }), jsx("div", {
        ref: div => this.chartContainer = div
      })]
    });
  }
}
CustomChart.propTypes = {
  type: PropTypes.string,
  colorsSet: PropTypes.string,
  chartConfig: PropTypes.object,
  data: PropTypes.array
};

export { CustomChart as default };
